﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using System.Windows.Forms;
using Microsoft.Win32;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the boot configuration.
    /// </summary>
    public static class Boot
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_BootConfiguration");

        /// <summary>
        /// Returns the current boot mode of this Windows session. 
        /// </summary>
        public static Helper.BootMode BootMode
        {
            get
            {
                Helper.BootMode bootMode;
                switch(SystemInformation.BootMode)
                {
                    case System.Windows.Forms.BootMode.Normal:
                        bootMode = Helper.BootMode.Normal;
                        break;
                    case System.Windows.Forms.BootMode.FailSafe:
                        bootMode = Helper.BootMode.FailSafe;
                        break;
                    case System.Windows.Forms.BootMode.FailSafeWithNetwork:
                        bootMode = Helper.BootMode.FailSafeNetwork;
                        break;
                    default:
                        bootMode = Helper.BootMode.Unknown;
                        break;
                }
                return bootMode;
            }
        }

        /// <summary>
        /// Returns the current boot device.
        /// </summary>
        public static string BootDevice
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SYSTEM\\CurrentControlSet\\Control", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("SystemBootDevice").ToString();
            }
        }

        /// <summary>
        /// Returns the path, where the boot configuration is saved.
        /// </summary>
        public static string ConfigurationPath
        {
            get
            {
                string configPath = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    configPath = queryObj["ConfigurationPath"].ToString();
                }
                return configPath;
            }
        }

        /// <summary>
        /// Returns the current boot directory.
        /// </summary>
        public static string BootDirectory
        {
            get
            {
                string bootDir = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    bootDir = queryObj["BootDirectory"].ToString();
                }
                return bootDir;
            }
        }

        /// <summary>
        /// Returns the current scratch directory.
        /// </summary>
        public static string ScratchDirectory
        {
            get
            {
                string scratchDir = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    scratchDir = queryObj["ScratchDirectory"].ToString();
                }
                return scratchDir;
            }
        }

        /// <summary>
        /// Returns the current temporary directory.
        /// </summary>
        public static string TempDirectory
        {
            get
            {
                string tempDir = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    tempDir = queryObj["TempDirectory"].ToString();
                }
                return tempDir;
            }
        }
    }
}
