﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.IO;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the network connection.
    /// </summary>
    public static class Network
    {
        /// <summary>
        /// Returns the connection state.
        /// </summary>
        public static bool NetworkConnected
        {
            get
            {
                return NetworkInterface.GetIsNetworkAvailable();
            }
        }

        /// <summary>
        /// Returns your local ip address.
        /// </summary>
        public static string LocalIP
        {
            get
            {
                string intip = Dns.GetHostAddresses(Dns.GetHostName()).First(a => a.AddressFamily == AddressFamily.InterNetwork).ToString();
                return intip;
            }
        }

        /// <summary>
        /// Returns your public ip address.
        /// </summary>
        public static string PublicIP
        {
            get
            {
                WebRequest req = HttpWebRequest.Create("http://checkip.dyndns.org/");
                WebResponse resp = req.GetResponse();
                Stream s = resp.GetResponseStream(); req = null; resp = null;
                StreamReader sr = new StreamReader(s);
                string ip = sr.ReadToEnd().Split(':')[1];
                ip = ip.Replace("</body>", "").Replace("</html>", "").Trim();
                return ip;
            }
        }

        /// <summary>
        /// Returns a <see cref="NetworkInterface"/>-Array containing information about the network interfaces.
        /// </summary>
        public static NetworkInterface[] EnumerateInterfaces(Helper.Protocols supportedProtocols)
        {
            List<NetworkInterface> interfaces = new List<NetworkInterface>();
            if (supportedProtocols == Helper.Protocols.IPv4)
            {
                foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
                {
                    if (ni.Supports(NetworkInterfaceComponent.IPv4) && !ni.Supports(NetworkInterfaceComponent.IPv6))
                    {
                        interfaces.Add(ni);
                    }
                }
            }
            else if (supportedProtocols == Helper.Protocols.IPv6)
            {
                foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
                {
                    if (!ni.Supports(NetworkInterfaceComponent.IPv4) && ni.Supports(NetworkInterfaceComponent.IPv6))
                    {
                        interfaces.Add(ni);
                    }
                }
            }
            else if (supportedProtocols == Helper.Protocols.AllProtocols)
            {
                foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
                {
                    if (ni.Supports(NetworkInterfaceComponent.IPv4) && ni.Supports(NetworkInterfaceComponent.IPv6))
                    {
                        interfaces.Add(ni);
                    }
                }
            }
            else if (supportedProtocols == Helper.Protocols.AllInterfaces)
            {
                foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
                {
                    interfaces.Add(ni);
                }
            }

            return interfaces.ToArray();
        }
    }
}
