﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the installed Microsoft Office version.
    /// </summary>
    public struct OfficeInfo
    {
        private static RegistryKey BaseKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
        private static RegistryKey SubKey = null;

        /// <summary>
        /// Returns an <see cref="OfficeInfo"/>-Array containing information about the installed Office versions.
        /// </summary>
        /// <returns></returns>
        public static OfficeInfo[] EnumerateOfficeVersions()
        {
            List<OfficeInfo> infos = new List<OfficeInfo>();

            {
                //Microsoft Office 2007 (12.0)
                RegistryKey regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
                regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\12.0\\Registration\\{90120000-0030-0000-0000-0000000FF1CE}", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey != null)
                {
                    OfficeInfo info = new OfficeInfo();
                    info.ProductName = regKey.GetValue("ProductName").ToString();
                    info.ServicePack = regKey.GetValue("SPLevel").ToString();
                    info.ProductID = regKey.GetValue("ProductID").ToString();
                    info.ProductKey = to2007Key((byte[])regKey.GetValue("DigitalProductID"));
                    infos.Add(info);
                }
            }

            {
                //Microsoft Office 2010 (14.0)
                RegistryKey regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
                regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\14.0\\Registration\\{91140000-0011-0000-0000-0000000FF1CE}",
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey != null)
                {
                    OfficeInfo info = new OfficeInfo();
                    info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                    info.ServicePack = regKey.GetValue("SPLevel").ToString();
                    info.ProductID = regKey.GetValue("ProductID").ToString();
                    info.ProductKey = to2010Key((byte[])regKey.GetValue("DigitalProductID"));
                    infos.Add(info);
                }

                if (Windows.Architecture == Helper.Architecture.x64)
                {
                    regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
                    regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\14.0\\Registration" +
                        "\\{91140000-0011-0000-0000-0000000FF1CE}", RegistryKeyPermissionCheck.ReadSubTree);
                    if (regKey != null)
                    {
                        OfficeInfo info = new OfficeInfo();
                        info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                        info.ServicePack = regKey.GetValue("SPLevel").ToString();
                        info.ProductID = regKey.GetValue("ProductID").ToString();
                        info.ProductKey = to2010Key((byte[])regKey.GetValue("DigitalProductID"));
                        infos.Add(info);
                    }
                }
            }

            {
                //Microsoft Office 2013 (15.0)
                RegistryKey regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
                regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\15.0\\Registration\\{91150000-0011-0000-1000-0000000FF1CE}",
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey != null)
                {
                    OfficeInfo info = new OfficeInfo();
                    info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                    info.ServicePack = regKey.GetValue("SPLevel").ToString();
                    info.ProductID = regKey.GetValue("ProductID").ToString();
                    info.ProductKey = to2013Key((byte[])regKey.GetValue("DigitalProductID"));
                    infos.Add(info);
                }

                if (Windows.Architecture == Helper.Architecture.x64)
                {
                    regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
                    regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\15.0\\Registration" +
                        "\\{91150000-0011-0000-1000-0000000FF1CE}", RegistryKeyPermissionCheck.ReadSubTree);
                    if (regKey != null)
                    {
                        OfficeInfo info = new OfficeInfo();
                        info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                        info.ServicePack = regKey.GetValue("SPLevel").ToString();
                        info.ProductID = regKey.GetValue("ProductID").ToString();
                        info.ProductKey = to2013Key((byte[])regKey.GetValue("DigitalProductID"));
                        infos.Add(info);
                    }
                }
            }

            {
                //Microsoft Office 2016 (16.0)                
                RegistryKey regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
                regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\ClickToRun\\REGISTRY\\MACHINE\\Software\\Microsoft\\Office" + 
                    "\\16.0\\Registration\\{90160000-000F-0000-1000-0000000FF1CE}", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey != null)
                {
                    OfficeInfo info = new OfficeInfo();
                    info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                    info.ServicePack = regKey.GetValue("SPLevel").ToString();
                    info.ProductID = regKey.GetValue("ProductID").ToString();
                    info.ProductKey = "Zurzeit nicht verfügbar"; // to2013Key((byte[])regKey.GetValue("DigitalProductID"));
                    infos.Add(info);
                }

                if (Windows.Architecture == Helper.Architecture.x64)
                {
                    regKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
                    regKey = regKey.OpenSubKey("SOFTWARE\\Microsoft\\Office\\ClickToRun\\REGISTRY\\MACHINE\\Software\\Microsoft" + 
                        "\\Office\\16.0\\Registration\\{90160000-000F-0000-1000-0000000FF1CE}", RegistryKeyPermissionCheck.ReadSubTree);
                    if (regKey != null)
                    {
                        OfficeInfo info = new OfficeInfo();
                        info.ProductName = regKey.GetValue("ConvertToEdition").ToString();
                        info.ServicePack = regKey.GetValue("SPLevel").ToString();
                        info.ProductID = regKey.GetValue("ProductID").ToString();
                        info.ProductKey = "Zurzeit nicht verfügbar"; // to2013Key((byte[])regKey.GetValue("DigitalProductID"));
                        infos.Add(info);
                    }
                }
            }

            return infos.ToArray();
        }

        /// <summary>
        /// The product name.
        /// </summary>
        public string ProductName { get; set; }
        /// <summary>
        /// The service pack.
        /// </summary>
        public string ServicePack { get; set; }
        /// <summary>
        /// The product ID.
        /// </summary>
        public string ProductID { get; set; }
        /// <summary>
        /// The product key.
        /// </summary>
        public string ProductKey { get; set; }

        #region ProductKey-Functions

        private static string to2007Key(byte[] rpk)
        {
            string strKey = "";
            const int iRPKOffset = 52;
            const string strPossibleChars = "BCDFGHJKMPQRTVWXY2346789";
            int i = 28;
            do
            {
                long lAccu = 0;
                int j = 14;
                do
                {
                    lAccu *= 256;
                    lAccu += System.Convert.ToInt64(rpk[iRPKOffset + j]);
                    rpk[iRPKOffset + j] = System.Convert.ToByte(System.Convert.ToInt64(Math.Floor((float)(lAccu) / 24.0F)) & Convert.ToInt64(255));
                    lAccu = lAccu % 24;
                    j -= 1;
                }
                while (j >= 0);
                i -= 1;
                strKey = strPossibleChars[(int)(lAccu)].ToString() + strKey;
                if ((0 == ((29 - i) % 6)) && !(-1 == i))
                {
                    i -= 1;
                    strKey = "-" + strKey;
                }
            }
            while (i >= 0);
            return strKey;
        }

        private static string to2010Key(byte[] rpk)
        {
            string KeyOutput = "";
            const int KeyOffset = 808;
            long isWin8 = (rpk[822] / 6) & 1;
            rpk[822] = Convert.ToByte((rpk[822] & 0xF7) | ((isWin8 & 2) * 4));
            int i = 24;
            string chars = "BCDFGHJKMPQRTVWXY2346789";
            int Last = 0;
            do
            {
                int Cur = 0;
                int X = 14;
                do
                {
                    Cur = Cur * 256;
                    Cur = rpk[X + KeyOffset] + Cur;
                    rpk[X + KeyOffset] = (byte)((Cur / 24));
                    Cur = Cur % 24;
                    X = X - 1;
                }
                while (X >= 0);
                i = i - 1;
                KeyOutput = chars.Substring(Cur + 1, 1) + KeyOutput;
                Last = Cur;
            }
            while (i >= 0);
            if (isWin8 == 1)
            {
                string keypart1 = KeyOutput.Substring(2, Last);
                string insert = "N";
                KeyOutput = Microsoft.VisualBasic.Strings.Replace(KeyOutput, keypart1, keypart1 + insert, 2, 1, 0);
                if (Last == 0)
                {
                    KeyOutput = insert + KeyOutput;
                }
            }
            string a = KeyOutput.Substring(1, 5);
            string b = KeyOutput.Substring(6, 5);
            string c = KeyOutput.Substring(11, 5);
            string d = KeyOutput.Substring(16, 5);
            string e = KeyOutput.Substring(21, 5);
            string prodkey = a + "-" + b + "-" + c + "-" + d + "-" + e;
            return prodkey;
        }

        private static string to2013Key(IList<Byte> digitalProductId)
        {

            int keyStartIndex = 52;
            int keyEndIndex = keyStartIndex + 15;

            const int numLetters = 24;
            // Possible alpha-numeric characters in product key.
            char[] digits = new[]
            {
                'B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'M', 'P', 'Q', 'R',
                'T', 'V', 'W', 'X', 'Y', '2', '3', '4', '6', '7', '8', '9'
            };

            // Check if Windows 8/Office 2013 Style Key (Can contain the letter "N")
            int containsN = (digitalProductId[keyStartIndex + 14] >> 3) & 1;
            digitalProductId[keyStartIndex + 14] = (byte)((digitalProductId[keyStartIndex + 14] & 0xF7) | ((containsN & 2) << 2));

            // Length of decoded product key
            const int decodeLength = 29;

            // Length of decoded product key in byte-form.
            // Each byte represents 2 chars.
            const int decodeStringLength = 15;

            // Array of containing the decoded product key.
            char[] decodedChars = new char[decodeLength];

            // Extract byte 52 to 67 inclusive.
            List<byte> hexPid = new List<byte>();
            for (int i = keyStartIndex; i <= keyEndIndex; i++)
            {
                hexPid.Add(digitalProductId[i]);
            }
            for (int i = decodeLength - 1; i >= 0; i--)
            {
                // Every sixth char is a separator.
                if ((i + 1) % 6 == 0)
                {
                    decodedChars[i] = '-';
                }
                else
                {
                    // Do the actual decoding.
                    int digitMapIndex = 0;
                    for (int j = decodeStringLength - 1; j >= 0; j--)
                    {
                        int byteValue = (digitMapIndex << 8) | hexPid[j];
                        hexPid[j] = (byte)(byteValue / numLetters);
                        digitMapIndex = byteValue % numLetters;
                        decodedChars[i] = digits[digitMapIndex];
                    }
                }
            }
            // Remove first character and put N in the right place
            if (containsN != 0)
            {
                int firstLetterIndex = 0;
                for (int index = 0; index < numLetters; index++)
                {
                    if (decodedChars[0] != digits[index]) continue;
                    firstLetterIndex = index;
                    break;
                }
                string keyWithN = new string(decodedChars);

                keyWithN = keyWithN.Replace("-", string.Empty).Remove(0, 1);
                keyWithN = keyWithN.Substring(0, firstLetterIndex) + "N" +
                                keyWithN.Remove(0, firstLetterIndex);
                keyWithN = keyWithN.Substring(0, 5) + "-" + keyWithN.Substring(5, 5) + "-" +
                                keyWithN.Substring(10, 5) + "-" + keyWithN.Substring(15, 5) + "-" +
                                keyWithN.Substring(20, 5);

                return keyWithN;
            }
            return new string(decodedChars);
        }

        #endregion
    }
}
