﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the installed sound devices.
    /// </summary>
    public struct SoundDevice
    {
        /// <summary>
        /// Returns a <see cref="SoundDevice"/>-Array with the installed sound devices.
        /// </summary>
        public static SoundDevice[] EnumerateDevices()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_SoundDevice");
            List<SoundDevice> devs = new List<SoundDevice>();
            foreach (ManagementObject queryObj in searcher.Get())
            {
                SoundDevice sDev = new SoundDevice();
                sDev.Name = queryObj["Name"].ToString();
                sDev.Manufacturer = queryObj["Manufacturer"].ToString();
                sDev.Status = queryObj["Status"].ToString();
                devs.Add(sDev);
            }
            return devs.ToArray();
        }

        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The manufacturer.
        /// </summary>
        public string Manufacturer { get; set; }
        /// <summary>
        /// The status.
        /// </summary>
        public string Status { get; set; }
    }
}
