﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using Microsoft.Win32;
using System.Drawing;
using System.Drawing.Imaging;
using System.Net;
using System.Reflection;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize the login procedure.
    /// </summary>
    public static class Logon
    {
        private static string winInstallPath = Environment.GetFolderPath(Environment.SpecialFolder.Windows);
        private static string finalPath = Path.Combine(winInstallPath, "System32", "oobe", "info", "backgrounds", "backgroundDefault.jpg");
        private static string backupPath = Path.Combine(winInstallPath, "System32", "oobe", "info", "backgrounds", "backgroundDefault_backup.jpg");

        /// <summary>
        /// Changes the logon background of Windows 7.
        /// </summary>
        /// <param name="bitmap"></param>
        /// <returns></returns>
        public static void ChangeLogonScreen(string bitmap)
        {
            string bmp = bitmap;

            if (!Directory.Exists(Path.Combine(winInstallPath, "System32", "oobe", "info"))) 
            { Directory.CreateDirectory(Path.Combine(winInstallPath, "System32", "oobe", "info")); }
            if (!Directory.Exists(Path.Combine(winInstallPath, "System32", "oobe", "info", "backgrounds"))) 
            { Directory.CreateDirectory(Path.Combine(winInstallPath, "System32", "oobe", "info", "backgrounds")); }

            if (File.Exists(finalPath))
            { File.Delete(backupPath); File.Move(finalPath, backupPath); }

            if (new FileInfo(bmp).Extension != ".jpg") 
            {
                string src = bmp;
                string dest = Path.Combine(Environment.CurrentDirectory, "backgroundDefault.jpg");
                ImageFormat nFormat = ImageFormat.Jpeg;
                Bitmap oBitmap = new Bitmap(src);
                oBitmap.Save(dest, nFormat);
                oBitmap.Dispose();
                bmp = dest;
            }

            CompressJPEG(bmp, 20);
            File.Copy(bmp, finalPath, true);

            RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI\\Background", true);
            regKey.SetValue("OEMBackground", 1, RegistryValueKind.DWord);
            return;
        }

        /// <summary>
        /// Compresses a JPEG image if necessary.
        /// </summary>
        /// <param name="bitmap"></param>
        /// <param name="quality"></param>
        private static void CompressJPEG(string bitmap, int quality) 
        {
            if (new FileInfo(bitmap).Length > 256000)
            {
                Image img = Image.FromFile(bitmap);
                ImageCodecInfo[] ImageCodecs;
                EncoderParameters ImageParameters;

                ImageCodecs = ImageCodecInfo.GetImageEncoders();
                ImageParameters = new EncoderParameters(1);
                ImageParameters.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, quality);

                File.Delete(bitmap);
                img.Save(bitmap, ImageCodecs[1], ImageParameters);
                img.Dispose();

                if (new FileInfo(bitmap).Length > 256000)
                { CompressJPEG(bitmap, quality); }
            }
            else
            { return; }
        }

        /// <summary>
        /// Changes the shade style.
        /// </summary>
        public static Helper.Style ShadeStyle
        {
            get
            {
                int ans = 0;
                Helper.Style ret = Helper.Style.NoShades;
                RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI", true);
                if (RegKey == null)
                { return Helper.Style.NoShades; }

                try { ans = int.Parse(RegKey.GetValue("ButtonSet").ToString()); }
                catch { }
                if (ans == 0)
                { ret = Helper.Style.NoShades; }
                else if (ans == 1)
                { ret = Helper.Style.LightShades; }
                else
                { ret = Helper.Style.DarkShades; }

                return ret;
            }
            set
            {
                try
                {
                    RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI", true);
                    switch (value)
                    {
                        case Helper.Style.DarkShades:
                            RegKey.SetValue("ButtonSet", 2, RegistryValueKind.DWord);
                            break;
                        case Helper.Style.LightShades:
                            RegKey.SetValue("ButtonSet", 1, RegistryValueKind.DWord);
                            break;
                        case Helper.Style.NoShades:
                            RegKey.SetValue("ButtonSet", 0, RegistryValueKind.DWord);
                            break;
                    }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Changes, whether Ctrl+Alt+Del is to be pressed before login.
        /// </summary>
        public static bool UseCtrlAltEntf
        {
            get
            {
                int ans = 0;
                bool ret = false;
                RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                if (RegKey == null)
                { return false; }

                try { ans = int.Parse(RegKey.GetValue("DisableCAD").ToString()); }
                catch { }
                if (ans == 0)
                { ret = true; }

                return ret;
            }
            set
            {
                try
                {
                    RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                    if (value)
                    { RegKey.SetValue("DisableCAD", 0, RegistryValueKind.DWord); }
                    else
                    { RegKey.SetValue("DisableCAD", 1, RegistryValueKind.DWord); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Gets or sets the message, which is shown before login.
        /// </summary>
        public static Helper.LogonMessage Message
        {
            get
            {
                try
                {
                    Helper.LogonMessage msg = new Helper.LogonMessage();
                    RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                    msg.Title = RegKey.GetValue("LegalNoticeCaption").ToString();
                    msg.Message = RegKey.GetValue("LegalNoticeText").ToString();
                    return msg;
                }
                catch (Exception ex)
                { throw ex; }
            }
            set
            {
                try
                {
                    RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                    RegKey.SetValue("LegalNoticeCaption", value.Title, RegistryValueKind.String);
                    RegKey.SetValue("LegalNoticeText", value.Message, RegistryValueKind.String);
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Sets, whether the on-screen keyboard is shown on tablets and touch-devices.
        /// </summary>
        public static bool TabletKeyboard
        {
            get
            {
                int ans = 0;
                bool ret = false;
                RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI", true);
                if (RegKey == null)
                { return false; }

                try { ans = int.Parse(RegKey.GetValue("ShowTabletKeyboard").ToString()); }
                catch { }
                if (ans == 1)
                { ret = true; }

                return ret;
            }
            set
            {
                try
                {
                    RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI", true);
                    if (value)
                    { RegKey.SetValue("ShowTabletKeyboard", 1, RegistryValueKind.DWord); }
                    else
                    { RegKey.SetValue("ShowTabletKeyboard", 0, RegistryValueKind.DWord); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Restores the original configuration.
        /// </summary>
        /// <returns></returns>
        public static void RestoreDefault()
        {
            try
            {
                RegistryKey RegKey1 = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI", true);
                RegistryKey RegKey2 = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI\\Background", true);
                RegistryKey RegKey3 = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                RegKey2.SetValue("OEMBackground", 0, RegistryValueKind.DWord);
                RegKey1.SetValue("ButtonSet", 0, RegistryValueKind.DWord);
                RegKey3.SetValue("DisableCAD", 1, RegistryValueKind.DWord);
                RegKey3.SetValue("LegalNoticeCaption", "", RegistryValueKind.String);
                RegKey3.SetValue("LegalNoticeText", "", RegistryValueKind.String);
                RegKey1.SetValue("ShowTabletKeyboard", 0, RegistryValueKind.DWord);

                return;
            }
            catch (Exception ex)
            { throw ex; }
        }

        /// <summary>
        /// Restores the backup image.
        /// </summary>
        public static void RestoreBackup()
        {
            if (!File.Exists(backupPath))
            { throw new Exception("No backup image available."); }

            string tempPath = Path.Combine(Path.GetDirectoryName(backupPath), "backgroundDefault_temp.jpg");

            File.Move(finalPath, tempPath);
            File.Move(backupPath, finalPath);
            File.Move(tempPath, backupPath);
        }

        /// <summary>
        /// Sets and gets the required information for auto login.
        /// </summary>
        public static bool AutoLogin
        {
            get
            {
                int ans = 0;
                bool ret = false;
                RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                if (RegKey == null)
                { return false; }

                try { ans = int.Parse(RegKey.GetValue("AutoAdminLogon").ToString()); }
                catch { }
                if (ans == 1)
                { ret = true; }

                return ret;
            }
            set
            {
                RegistryKey RegKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", true);
                switch (value)
                {
                    case true:
                        NetworkCredential netcred = new NetworkCredential();
                        CredentialDialog.Show("Automatic Login", "Please type in your login information, which is to be used to log you in automatically.", ref netcred, null);
                        try
                        {
                            RegKey.SetValue("AutoAdminLogon", "1");
                            RegKey.SetValue("DefaultUsername", netcred.UserName);
                            RegKey.SetValue("DefaultPassword", netcred.Password);
                        }
                        catch (Exception ex)
                        { throw ex; }
                        break;
                    case false:
                        try
                        {
                            RegKey.SetValue("AutoAdminLogon", "0");
                            RegKey.SetValue("DefaultUsername", "");
                            RegKey.SetValue("DefaultPassword", "");
                        }
                        catch (Exception ex)
                        { throw ex; }
                        break;
                }
            }
        }
    }
}
