﻿using System;
using System.Security.AccessControl;
using Microsoft.Win32;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize the new Modern UI.
    /// </summary>
    public static class ModernUI
    {
        #region Windows 10

        /// <summary>
        /// Enables/disables the "Hero" wallpaper on logon screen.
        /// </summary>
        public static bool ColorOnLogonScreen
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\System");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("DisableLogonBackgroundImage").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\System", true);
                if (value)
                { regKey.SetValue("DisableLogonBackgroundImage", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("DisableLogonBackgroundImage", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the dark theme for Modern apps.
        /// </summary>
        public static bool DarkMode
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("AppsUseLightTheme").ToString()); }
                catch { return false; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize", true);
                if (value)
                { regKey.SetValue("AppsUseLightTheme", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("AppsUseLightTheme", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the usage of the old volume control UI.
        /// </summary>
        public static bool LegacyVolumeUI
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\MTCUVC");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EnableMtcUvc").ToString()); }
                catch { return false; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\MTCUVC", true);
                if (regKey == null)
                { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", true)
                          .CreateSubKey("MTCUVC"); }
                if (value)
                { regKey.SetValue("EnableMtcUvc", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EnableMtcUvc", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Configures the UI of the network settings.
        /// </summary>
        public static Helper.NetworkUI NetworkUI
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Control Panel\\Settings\\Network", RegistryKeyPermissionCheck.ReadSubTree);

                if (regKey == null)
                { return Helper.NetworkUI.Default; }

                try { ans = int.Parse(regKey.GetValue("ReplaceVan").ToString()); }
                catch { return Helper.NetworkUI.Default; }
                if (ans == 0)
                { return Helper.NetworkUI.Default; }
                else if (ans == 1)
                { return Helper.NetworkUI.NetworkSettings; }
                else if (ans == 2)
                { return Helper.NetworkUI.Windows8Sidebar; }
                else
                { return Helper.NetworkUI.Default; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion" +
                    "\\Control Panel\\Settings\\Network", RegistryKeyPermissionCheck.ReadWriteSubTree);

                //string identity = Environment.UserDomainName + "\\" + Environment.UserName;
                //RegistrySecurity regSec = new RegistrySecurity();
                //regSec.SetAccessRule(new RegistryAccessRule(identity, RegistryRights.FullControl, AccessControlType.Allow));
                //regKey.SetAccessControl(regSec);

                if (value == Helper.NetworkUI.Default)
                { regKey.SetValue("ReplaceVan", 0, RegistryValueKind.DWord); }
                else if (value == Helper.NetworkUI.NetworkSettings)
                { regKey.SetValue("ReplaceVan", 1, RegistryValueKind.DWord); }
                else if (value == Helper.NetworkUI.Windows8Sidebar)
                { regKey.SetValue("ReplaceVan", 2, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("ReplaceVan", 0, RegistryValueKind.DWord); }

            }
        } //PERMISSIONS STILL NEEDED MANUALLY

        /// <summary>
        /// Enables/disables the old calendar UI.
        /// </summary>
        public static bool LegacyCalendarUI
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("UseWin32TrayClockExperience").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true);
                if (value)
                { regKey.SetValue("UseWin32TrayClockExperience", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("UseWin32TrayClockExperience", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables the old battery UI.
        /// </summary>
        public static bool LegacyBatteryUI
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("UseWin32BatteryFlyout").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true);
                if (value)
                { regKey.SetValue("UseWin32BatteryFlyout", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("UseWin32BatteryFlyout", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the notification center UI of Windows 10 preview builds.
        /// </summary>
        public static bool FormerNCUI
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("UseActionCenterExperience").ToString()); }
                catch { return false; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true);
                if (value)
                { regKey.SetValue("UseActionCenterExperience", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("UseActionCenterExperience", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the action center.
        /// </summary>
        public static bool ActionCenter
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Explorer");
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("DisableNotificationCenter").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Explorer", true);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows", true).CreateSubKey("Explorer");
                }

                if (value)
                { regKey.SetValue("DisableNotificationCenter", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("DisableNotificationCenter", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the balloon notifications.
        /// </summary>
        public static bool BalloonNotifications
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Explorer");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EnableLegacyBalloonNotifications").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Explorer", true);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows", true).CreateSubKey("Explorer");
                }
                if (value)
                { regKey.SetValue("EnableLegacyBalloonNotifications", 1, RegistryValueKind.DWord); }
            }
        }

        #region Folders in 'This PC'

        /// <summary>
        /// Shows/hides the Desktop folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Desktop
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{B4BFCC3A-DB2C-424C-B029-7FE99A87C641}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return true; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{B4BFCC3A-DB2C-424C-B029-7FE99A87C641}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Shows/hides the Pictures folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Pictures
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{0ddd015d-b06c-45d5-8c4c-f59713854639}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return false; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{0ddd015d-b06c-45d5-8c4c-f59713854639}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Shows/hides the Videos folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Videos
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{35286a68-3c57-41a1-bbb1-0eae73d76c95}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return false; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{35286a68-3c57-41a1-bbb1-0eae73d76c95}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Shows/hides the Downloads folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Downloads
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{7d83ee9b-2244-4e70-b1f5-5393042af1e4}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return false; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{7d83ee9b-2244-4e70-b1f5-5393042af1e4}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Shows/hides the Music folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Music
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{a0c69a99-21c8-4671-8703-7934162fcf1d}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return false; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{a0c69a99-21c8-4671-8703-7934162fcf1d}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Shows/hides the Documents folder in 'This PC'.
        /// </summary>
        public static bool ThisPC_Documents
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{f42ee2d3-909f-4907-8871-4c22fc0bf756}\\PropertyBag");
                if (regKey == null)
                { return false; }

                try { ans = regKey.GetValue("ThisPCPolicy").ToString(); }
                catch { return false; }

                if (ans == "Show")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FolderDescriptions" +
                    "\\{f42ee2d3-909f-4907-8871-4c22fc0bf756}\\PropertyBag", true);
                if (value)
                { regKey.SetValue("ThisPCPolicy", "Show", RegistryValueKind.String); }
                else
                { regKey.SetValue("ThisPCPolicy", "Hide", RegistryValueKind.String); }
            }
        }

        #endregion

        //PERMISSIONS STILL NEEDED MANUALLY
        #region Items in Navigation Pane 

        /// <summary>
        /// Shows/hides the OneDrive-Item in the Explorer's Navigation Pane.
        /// </summary>
        public static bool NavigationPane_OneDrive
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{018D5C66-4533-4307-9B53-224DE2ED1FE6}");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("System.IsPinnedToNameSpaceTree").ToString()); }
                catch { return false; }

                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{018D5C66-4533-4307-9B53-224DE2ED1FE6}", true);
                if (value)
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Shows/hides the QuickAccess-Item in the Explorer's Navigation Pane.
        /// </summary>
        public static bool NavigationPane_QuickAccess
        {
            get
            {
                long ans = 0;
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{679f85cb-0220-4080-b29b-5540cc05aab6}\\ShellFolder", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = long.Parse(regKey.GetValue("Attributes").ToString()); }
                catch { return false; }

                if (ans == 0xa0100000)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{679f85cb-0220-4080-b29b-5540cc05aab6}\\ShellFolder", true);
                if (value)
                { regKey.SetValue("Attributes", 0xa0100000, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("Attributes", 0xa0600000, RegistryValueKind.DWord); }
            }
        } //Permissions still needed manually

        /// <summary>
        /// Shows/hides the Homegroup-Item in the Explorer's Navigation Pane.
        /// </summary>
        public static bool NavigationPane_Homegroup
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{B4FB3F98-C1EA-428d-A78A-D1F5659CBA93}");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("System.IsPinnedToNameSpaceTree").ToString()); }
                catch { return false; }

                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{B4FB3F98-C1EA-428d-A78A-D1F5659CBA93}", true);
                if (value)
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 0, RegistryValueKind.DWord); }
            }
        } //Permissions still needed manually

        /// <summary>
        /// Shows/hides the Network-Item in the Explorer's Navigation Pane.
        /// </summary>
        public static bool NavigationPane_Network
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{F02C1A0D-BE21-4350-88B0-7367FC96EF3C}");
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("System.IsPinnedToNameSpaceTree").ToString()); }
                catch { return false; }

                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("CLSID\\{F02C1A0D-BE21-4350-88B0-7367FC96EF3C}", true);
                if (value)
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("System.IsPinnedToNameSpaceTree", 0, RegistryValueKind.DWord); }
            }
        } //Permissions still needed manually

        /// <summary>
        /// Shows/hides the items for removable drives in the Explorer's Navigation Pane.
        /// </summary>
        public static bool NavigationPane_RemovableDrives
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Desktop" 
                    + "\\NameSpace\\DelegateFolders\\{F5FB2C77-0E2F-4A16-A381-3E560C68BC83}");
                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Desktop"
                    + "\\NameSpace\\DelegateFolders", true);
                if (value)
                { regKey.CreateSubKey("{F5FB2C77-0E2F-4A16-A381-3E560C68BC83}"); }
                else
                { regKey.DeleteSubKeyTree("{F5FB2C77-0E2F-4A16-A381-3E560C68BC83}", false); }
            }
        }

        #endregion

        /// <summary>
        /// Enables/disables OneDrive.
        /// </summary>
        public static bool OneDrive
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\OneDrive");
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("DisableFileSyncNGSC").ToString()); }
                catch { return true; }
                if (ans == 1)
                { return false; }
                else
                { return true; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\OneDrive", true);
                if (regKey == null)
                { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows", true).CreateSubKey("OneDrive"); }
                if (value)
                { regKey.SetValue("DisableFileSyncNGSC", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("DisableFileSyncNGSC", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the transparency of the taskbar.
        /// </summary>
        public static bool TaskbarTransparency
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize");
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("EnableTransparency").ToString()); }
                catch { return true; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize", true);
                if (value)
                { regKey.SetValue("EnableTransparency", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EnableTransparency", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Uses the OLED optimization to improve the transparency of the taskbar.
        /// </summary>
        public static bool OLEDTransparentTaskbar
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Advanced");
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("UseOLEDTaskbarTransparency").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Advanced", true);
                if (value)
                { regKey.SetValue("UseOLEDTaskbarTransparency", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("UseOLEDTaskbarTransparency", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the translucency of the taskbar.
        /// </summary>
        public static bool TaskbarTranslucency
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize");
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("EnableBlurBehind").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize", true);
                if (value)
                { regKey.SetValue("EnableBlurBehind", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EnableBlurBehind", 0, RegistryValueKind.DWord); }
            }
        }

        #endregion

        #region Windows 8.1

        /// <summary>
        /// Enables/disables booting directly to the desktop.
        /// </summary>
        public static bool BootToDesktop
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("OpenAtLogon").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (value)
                { regKey.SetValue("OpenAtLogon", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("OpenAtLogon", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables showing all installed apps on the Start screen.
        /// </summary>
        public static bool AllAppsOnStart
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("MakeAllAppsDefault").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (value)
                { regKey.SetValue("MakeAllAppsDefault", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("MakeAllAppsDefault", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables, whether the desktop apps will be shown at first.
        /// </summary>
        public static bool DesktopAppsFirst
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("DesktopFirst").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StartPage", true);
                if (value)
                { regKey.SetValue("DesktopFirst", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("DesktopFirst", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the usage of the desktop background on the Start screen.
        /// </summary>
        public static bool DesktopBackgroundOnStart
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Accent", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("MotionAccentId_v1.00").ToString()); }
                catch { return false; }
                if (ans == 0x000000db)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Accent", true);
                if (value)
                { regKey.SetValue("MotionAccentId_v1.00", 0x000000db, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("MotionAccentId_v1.00", 0x000000dd, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the backup of Windows apps.
        /// </summary>
        public static bool WindowsAppsBackup
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\SettingSync", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EnableBackupForWin8Apps").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\SettingSync", true);
                if (value)
                { regKey.SetValue("EnableBackupForWin8Apps", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EnableBackupForWin8Apps", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Returns the root directory for Windows apps or sets it.
        /// </summary>
        public static string WindowsAppsInstallRoot
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Appx");
                return regKey.GetValue("PackageRoot").ToString();
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Appx", true);
                regKey.SetValue("PackageRoot", value);
            }
        }

        /// <summary>
        /// Enables/disables shortcuts to the libraries on "Computer"
        /// </summary>
        public static bool LibrariesOnComputer
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace\\{B4BFCC3A-DB2C-424C-B029-7FE99A87C641}");
                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{B4BFCC3A-DB2C-424C-B029-7FE99A87C641}"); //Desktop
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{A8CDFF1C-4878-43be-B5FD-F8091C1C60D0}"); //Documents  
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{374DE290-123F-4565-9164-39C4925E467B}"); //Downloads
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{1CF1260C-4DD0-4ebb-811F-33C572699FDE}"); //Music
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{3ADD1653-EB32-4cb0-BBD7-DFA0ABB5ACCA}"); //Pictures
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).CreateSubKey("{A0953C92-50DC-43bf-BE83-3742FED03C9C}"); //Videos
                }
                else
                {
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{B4BFCC3A-DB2C-424C-B029-7FE99A87C641}"); //Desktop
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{A8CDFF1C-4878-43be-B5FD-F8091C1C60D0}"); //Documents  
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{374DE290-123F-4565-9164-39C4925E467B}"); //Downloads
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{1CF1260C-4DD0-4ebb-811F-33C572699FDE}"); //Music
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{3ADD1653-EB32-4cb0-BBD7-DFA0ABB5ACCA}"); //Pictures
                    Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\NameSpace", true).DeleteSubKey("{A0953C92-50DC-43bf-BE83-3742FED03C9C}"); //Videos
                }
            }
        }

        #endregion

        #region Windows 8

        /// <summary>
        /// Enables/disables the lock screen.
        /// </summary>
        public static bool EnableLockScreen
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoLockScreen").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                    if (regKey == null)
                    {
                        Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows", true).CreateSubKey("Personalization");
                        regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                    }

                    if (value)
                    { regKey.SetValue("NoLockScreen", 0); }
                    else
                    { regKey.SetValue("NoLockScreen", 1); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables the CharmBar.
        /// </summary>
        public static bool EnableCharmBar
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("DisableCharmsHint").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                    if (regKey == null)
                    {
                        Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("EdgeUI");
                        regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                    }

                    if (value)
                    { regKey.SetValue("DisableCharmsHint", 0, RegistryValueKind.DWord); }
                    else
                    { regKey.SetValue("DisableCharmsHint", 1, RegistryValueKind.DWord); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables the Hot Corners.
        /// </summary>
        public static bool EnableHotCorners
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("DisableTLcorner").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                    if (regKey == null)
                    {
                        Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("EdgeUI");
                        regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\EdgeUI", true);
                    }
                    if (value)
                    {
                        regKey.SetValue("DisableTLcorner", 0, RegistryValueKind.DWord);
                        regKey.SetValue("DisableTRcorner", 0, RegistryValueKind.DWord);
                    }
                    else
                    {
                        regKey.SetValue("DisableTLcorner", 1, RegistryValueKind.DWord);
                        regKey.SetValue("DisableTRcorner", 1, RegistryValueKind.DWord);
                    }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables the uninstall of apps from the Start screen.
        /// </summary>
        public static bool EnableUninstallFromStart
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Explorer", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoUninstallFromStart").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Explorer", true);
                    if (value)
                    { regKey.SetValue("NoUninstallFromStart", 0); }
                    else
                    { regKey.SetValue("NoUninstallFromStart", 1); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables changing the background of the Start screen.
        /// </summary>
        public static bool EnableChangingStartBackground
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoChangingStartMenuBackground").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                    if (regKey == null)
                    {
                        Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows", true).CreateSubKey("Personalization");
                        regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Policies\\Microsoft\\Windows\\Personalization", true);
                    }

                    if (value)
                    { regKey.SetValue("NoChangingStartMenuBackground", 0); }
                    else
                    { regKey.SetValue("NoChangingStartMenuBackground", 1); }
                }
                catch (Exception secex)
                { throw secex; }
            }
        }

        /// <summary>
        /// Sets the maximum number of rows on the Start screen.
        /// </summary>
        public static int MaxRowCount
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                if (regKey == null)
                { return 0; }

                try { ans = int.Parse(regKey.GetValue("Layout_MaximumRowCount").ToString()); }
                catch { }

                return ans;
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                    if (regKey == null)
                    {
                        Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("Grid");
                        regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                    }

                    regKey.SetValue("Layout_MaximumRowCount", value, RegistryValueKind.DWord);
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables the fade animation of the Start screen.
        /// </summary>
        public static bool StartScreenAnimation
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("Launcher_SessionLoginAnimation_OnShow").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                    if (regKey == null)
                    {
                        Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("Grid");
                        regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\Grid", true);
                    }

                    if (value)
                    { regKey.SetValue("Launcher_SessionLoginAnimation_OnShow", 1, RegistryValueKind.DWord); }
                    else
                    { regKey.SetValue("Launcher_SessionLoginAnimation_OnShow", 0, RegistryValueKind.DWord); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables Snap on low resolution.
        /// </summary>
        public static bool AllowSnap
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\AppPositionier", true);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("AlwaysEnableLSSnapping").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                try
                {
                    Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("AppPositionier");
                    RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\AppPositionier", true);
                    if (regKey == null)
                    {
                        Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell", true).CreateSubKey("AppPositionier");
                        regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\ImmersiveShell\\AppPositionier", true);
                    }

                    if (value)
                    { regKey.SetValue("AlwaysEnableLSSnapping", 1, RegistryValueKind.DWord); }
                    else
                    { regKey.SetValue("AlwaysEnableLSSnapping", 0, RegistryValueKind.DWord); }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Enables/disables the Windows Store.
        /// </summary>
        public static bool WindowsStore
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\WindowsStore", true);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("RemoveWindowsStore").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\WindowsStore", true);
                if (regKey == null)
                {
                    Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft", true).CreateSubKey("WindowsStore");
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\WindowsStore", true);
                }

                if (value)
                { regKey.SetValue("RemoveWindowsStore", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("RemoveWindowsStore", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Sets the state of the SmartScreen filter.
        /// </summary>
        public static Helper.SmartScreen SmartScreen
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer", true);
                if (regKey == null)
                { return Helper.SmartScreen.AdminApproval; }

                try { ans = regKey.GetValue("SmartScreenEnabled").ToString(); }
                catch { return Helper.SmartScreen.AdminApproval; }
                if (ans == "Prompt")
                { return Helper.SmartScreen.Warning; }
                else if (ans == "RequireAdmin")
                { return Helper.SmartScreen.AdminApproval; }
                else
                { return Helper.SmartScreen.TurnedOff; }
            }
            set
            {
                try
                {
                    RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer", true);
                    switch (value)
                    {
                        case Helper.SmartScreen.Warning:
                            regKey.SetValue("SmartScreenEnabled", "Prompt", RegistryValueKind.String);
                            break;
                        case Helper.SmartScreen.AdminApproval:
                            regKey.SetValue("SmartScreenEnabled", "RequireAdmin", RegistryValueKind.String);
                            break;
                        case Helper.SmartScreen.TurnedOff:
                            regKey.SetValue("SmartScreenEnabled", "Off", RegistryValueKind.String);
                            break;
                    }
                }
                catch (Exception ex)
                { throw ex; }
            }
        }

        /// <summary>
        /// Returns the image of the lock screen or sets it.
        /// </summary>
        public static string DefaultLockScreenImage
        {
            get
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Personalization", true);
                return regKey.GetValue("LockScreenImage").ToString();
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Personalization", true);
                if (regKey == null)
                {
                    Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows", true).CreateSubKey("Personalization");
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Personalization", true);
                }

                regKey.SetValue("LockScreenImage", value);
            }
        }

        #endregion
    }
}
