﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;
using System.Diagnostics;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize security settings.
    /// </summary>
    public static class Security
    {
        /// <summary>
        /// Configures the access to the command line.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static Helper.CMDAccess CommandLineAccess
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\System", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return Helper.CMDAccess.Full; }

                try { ans = int.Parse(regKey.GetValue("DisableCMD").ToString()); }
                catch { return Helper.CMDAccess.Full; }

                if (ans == 0)
                { return Helper.CMDAccess.Full; }
                else if (ans == 1)
                { return Helper.CMDAccess.OnlyBatch; }
                else
                { return Helper.CMDAccess.None; }
            }
            set
            { 
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\System", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("System", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                }

                if (value == Helper.CMDAccess.Full)
                { regKey.SetValue("DisableCMD", 0, RegistryValueKind.DWord); }
                else if (value == Helper.CMDAccess.OnlyBatch)
                { regKey.SetValue("DisableCMD", 1, RegistryValueKind.DWord); }
                else if (value == Helper.CMDAccess.None)
                { regKey.SetValue("DisableCMD", 2, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Changes the state of the firewall.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool Firewall
        {
            set
            {
                Process proc = new Process();
                proc.StartInfo.CreateNoWindow = true;
                proc.StartInfo.UseShellExecute = false;
                proc.StartInfo.FileName = "cmd.exe";
                if (value)
                {
                    proc.StartInfo.Arguments = "/c netsh advfirewall set currentprofile state on";
                    proc.Start();
                }
                else
                {
                    proc.StartInfo.Arguments = "/c netsh advfirewall set currentprofile state off";
                    proc.Start();
                }
            }
        }

        /// <summary>
        /// Changes the state of user account control.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool UserAccountControl
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Policies" + 
                    "\\System", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EnableLUA").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Policies" + 
                    "\\System", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("EnableLUA", 1); }
                else
                { regKey.SetValue("EnableLUA", 0); }
            }
        }
    }
}
